""" Security functions for the server. """
import re
import time
import hash_calculator
import settings
from const import UTF8

def perform_hash_checks(str_request)->bool:
    """ Performs a hash check on the message, and verifies the timestamp is valid.
        If either check fails, return False.  Otherwise, return True. 
        
        Parameters:
            str_request: The request body, as a string.
        Returns:
            True if the message is valid, False otherwise."""
    my_hash = get_message_hash(str_request)

    if my_hash not in str_request:
        print("Error: hash not match")
        return False
    if not verify_message_time(str_request):
        print("Error: timestamp expired")
        return False
    return True

def get_message_hash(json_command) -> str:
    """ Get the object named "message", and run a hash over it. The hash is calculated 
        using the SHA512 algorithm. The hash is returned as a hex string.  
        Parameters:
            json_command: The JSON command to hash.
        Returns:
            The hash of the message, as a string. """

    strip1 = re.sub(".*\"message\":", "", json_command, 1)
    if "\"hash\":" in strip1:
        strip2 = re.sub(',\"hash\":.*', '', strip1)
    else:
        strip2 = strip1[:-1]
    json_value = bytes(strip2, UTF8)
    if settings.HASHKEY is not None:
        hash_value = hash_calculator.calculate_hash(
            json_value, settings.HASHKEY)
        return hash_value
    return ""


def verify_message_time(json_command) -> bool:
    """Check message expiration is less than current time + time in settings.
        Before we accept the JSON as valid, and parse it, we must check the timestamp.
        The timestamp is a Linux Timestamp. So convert it, add maximum message age, and
        then verify current time is less than expiration time.
    """
    strip_json = re.sub('.*\"timestamp\":', "", json_command, 1)
    expiration = int(re.sub('}.*', '', strip_json))+settings.MAX_MESSAGE_AGE
    current_time = int(time.time())
    return not current_time > expiration
